# This file was auto-generated by Fern from our API Definition.

import typing

IssueTag = typing.Union[
    typing.Literal[
        "Pointer", "Unexpected", "Missing", "Composite", "Refinement", "Transformation", "Type", "Forbidden"
    ],
    typing.Any,
]
