# This file was auto-generated by Fern from our API Definition.

import typing

TracesSearchRequestAttributeFiltersItemOperator = typing.Union[
    typing.Literal["eq", "neq", "contains", "exists"], typing.Any
]
