# This file was auto-generated by Fern from our API Definition.

import enum


class MirascopeEnvironment(enum.Enum):
    PRODUCTION = "https://v2.mirascope.com/api/v0"
    STAGING = "https://staging.mirascope.com/api/v0"
    LOCAL = "http://localhost:3000/api/v0"
