# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class TracesSearchResponseSpansItem(UniversalBaseModel):
    id: str
    trace_id: typing_extensions.Annotated[str, FieldMetadata(alias="traceId")]
    span_id: typing_extensions.Annotated[str, FieldMetadata(alias="spanId")]
    name: str
    start_time: typing_extensions.Annotated[str, FieldMetadata(alias="startTime")]
    duration_ms: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="durationMs")
    ] = None
    model: typing.Optional[str] = None
    provider: typing.Optional[str] = None
    total_tokens: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="totalTokens")
    ] = None
    function_id: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="functionId")
    ] = None
    function_name: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="functionName")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
