# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .annotations_create_response_label import AnnotationsCreateResponseLabel


class AnnotationsCreateResponse(UniversalBaseModel):
    id: str
    span_id: typing_extensions.Annotated[str, FieldMetadata(alias="spanId")]
    trace_id: typing_extensions.Annotated[str, FieldMetadata(alias="traceId")]
    otel_span_id: typing_extensions.Annotated[str, FieldMetadata(alias="otelSpanId")]
    otel_trace_id: typing_extensions.Annotated[str, FieldMetadata(alias="otelTraceId")]
    label: typing.Optional[AnnotationsCreateResponseLabel] = None
    reasoning: typing.Optional[str] = None
    metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = None
    environment_id: typing_extensions.Annotated[str, FieldMetadata(alias="environmentId")]
    project_id: typing_extensions.Annotated[str, FieldMetadata(alias="projectId")]
    organization_id: typing_extensions.Annotated[str, FieldMetadata(alias="organizationId")]
    created_by: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="createdBy")] = None
    created_at: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="createdAt")] = None
    updated_at: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="updatedAt")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
